@@ -1,15 +1,25 @@
-module(...,package.seeall)
-require "zip"
-xmlparser = require ("luaxml-mod-xml")
-handler = require("luaxml-mod-handler")
+-- Package odsfile. Author Michal Hoftich <michal.h21@gmail.com>
+-- This package is subject of LPPL license, version 1.3c
+-- module(...,package.seeall)
 
+local M = {}
+local zip = require "zip"
+local xmlparser = require ("luaxml-mod-xml")
+local handler = require("luaxml-mod-handler")
+
+local namedRanges = {}
 
 function load(filename)
+  -- add support for -reader command line option
+  -- we must open the file and close it immediatelly 
+  local f = io.open(filename, "r")
+  f:close()
   local p = {
     file = zip.open(filename),
     content_file_name = "content.xml",
     loadContent = function(self,filename)
       local treehandler = handler.simpleTreeHandler()
+      -- treehandler.options.noReduce = {["table:table-cell"]=true, ["text:p"]=true}
       local filename = filename or self.content_file_name  
       local xmlfile = self.file:open(filename)
       local text = xmlfile:read("*a")
@@ -28,6 +38,7 @@ function getTable(x,table_name)
   for key, val in pairs(t) do
     if key == "table:table-row" then
       local rows = {}
+      if #val == 0 then val = {val} end
       
       for i = 1, #val do
         local r = val[i]
@@ -36,6 +47,9 @@ function getTable(x,table_name)
         row = {}
         row["_attr"] = r["_attr"]
         local cc = r["table:table-cell"] or {}
+        if #cc == 0 then
+          cc = {cc}
+        end
         
         local columns = {}
         --for j = 1, #cc do
@@ -65,6 +79,7 @@ end
 
 function getTable0(x,table_name)
   local tables = x.root["office:document-content"]["office:body"]["office:spreadsheet"]["table:table"]
+  namedRanges = loadNameRanges(x, table_name)
   if #tables > 1 then
     if type(tables) == "table" and table_name ~= nil then 
       for k,v in pairs(tables) do
@@ -93,6 +108,32 @@ function getColumnCount(tbl)
   return x
 end
 
+function loadNameRanges(root, tblname)
+  local tblname = tblname or ""
+  local t = {}
+  local ranges = root.root["office:document-content"]["office:body"]["office:spreadsheet"]["table:named-expressions"]
+  if not ranges then return {} end
+  ranges = ranges["table:named-range"] or {}
+  if #ranges == 0 then 
+    ranges = {ranges}
+  end
+  for _,r in ipairs(ranges) do
+    local a = r["_attr"] or {}
+    local range = a["table:cell-range-address"] or ""
+    local name = a["table:name"] 
+    if name and range:match("^$?"..tblname) then
+      range = range:gsub("^[^%.]*",""):gsub("[%$%.]","")
+      print("named range", name, range)
+      t[name] = range
+    end
+  end
+  return t
+end
+
+
+
+
+
 function tableValues(tbl,x1,y1,x2,y2)
   local t= {}
   local x1 = x1 or 1
@@ -106,8 +147,9 @@ function tableValues(tbl,x1,y1,x2,y2)
       if #v["table:table-cell"] > 1 then
         local r = table_slice(v["table:table-cell"],x1,x2)
         for p,n in pairs(r) do
+          local attr = n["_attr"]
           local cellValue = n["text:p"] or ""
-            table.insert(j,{value=cellValue,attr=att})
+          table.insert(j,{value=cellValue,attr=attr})
         end
       else
         local p = {value=v["table:table-cell"]["text:p"],attr=v["table:table-cell"]["_attr"]} 
@@ -120,6 +162,8 @@ function tableValues(tbl,x1,y1,x2,y2)
 end
 
 function getRange(range)
+  if range == nil then return {nil,nil,nil,nil} end
+  local range = namedRanges[range] or range
   local r = range:lower()
   local function getNumber(s)
     if s == "" or s == nil then return nil end
@@ -131,7 +175,7 @@ function getRange(range)
     return f
   end
   for x1,y1,x2,y2 in r:gmatch("(%a*)(%d*):*(%a*)(%d*)") do
-    return getNumber(x1),tonumber(y1),getNumber(x2),tonumber(y2) 
+    return {getNumber(x1),tonumber(y1),getNumber(x2),tonumber(y2)}
    --print(string.format("%s, %s, %s, %s",getNumber(x1),y1,getNumber(x2),y2))
   end
 end
@@ -169,14 +213,26 @@ function interp(s, tab)
   )
 end
 
+function escape(s)
+  return string.gsub(s, "([%\\]?)([#%%%$&_%{%}%\\|])", function(a,b)
+    if a=="" then 
+      if b == "\\" then
+        return "\\textbackslash"
+      elseif b == "|" then
+        return "\\textbar"
+      else
+        return "\\"..b 
+      end
+    elseif a=="\\" and b=="\\" then
+        return "\\textbackslash\\textbackslash"
+    end
+  end)
+end
+
 get_link = function(val)
   local k = val["text:a"][1]
   local href = val["text:a"]["_attr"]["xlink:href"]
-  return "\\odslink{"..href.."}{"..k.."}"
-end
-
-function escape(s)
-  return string.gsub(s, "([#%%$&])", "\\%1")
+  return "\\odslink{"..href.."}{".. escape(k).."}"
 end
 
 
@@ -191,7 +247,8 @@ function get_cell(val, delim)
     elseif val["text:span"] then
       return get_cell(val["text:span"], delim)
     elseif val["text:s"] then
-      return get_cell(val["text:s"], delim)
+      -- return get_cell(val["text:s"], delim)
+      return escape(table.concat(val, " "))
     else
       local t = {}
       for _,v in ipairs(val) do
@@ -265,3 +322,29 @@ function updateZip(zipfile, updatefile)
   local command  =  string.format("zip %s %s",zipfile, updatefile)
   print ("Updating an ods file.\n" ..command .."\n Return code: ", os.execute(command))  
 end
+
+M.load= load
+M.loadContent  =    loadContent  
+M.getTable= getTable
+M.getTable0= getTable0
+M.getColumnCount= getColumnCount
+M.loadNameRanges= loadNameRanges
+M.tableValues= tableValues
+M.getRange= getRange
+M.getNumber=  getNumber
+M.table_slice = table_slice 
+M.interp= interp
+M.get_link  =get_link  
+M.escape= escape
+M.get_cell= get_cell
+M.newRow= newRow
+    -- Generic  for inserting cell
+M.addCell  =    addCell  
+M.addString  =    addString  
+M.addFloat  =    addFloat  
+M.findLastRow  =    findLastRow  
+M.insert  =    insert  
+--  for updateing the archive. Depends on external zip utility
+M.updateZip= updateZip
+
+return M
