@@ -7,8 +7,8 @@
 
 assert(luaotfload_module, "This is a part of luaotfload and should not be loaded independently") { 
     name          = "luaotfload-configuration",
-    version       = "3.16",       --TAGVERSION
-    date          = "2020-12-31", --TAGDATE
+    version       = "3.23",       --TAGVERSION
+    date          = "2022-10-03", --TAGDATE
     description   = "luaotfload submodule / config file reader",
     license       = "GPL v2.0"
 }
@@ -218,6 +218,7 @@ local default_config = {
     version        = luaotfload.version,
     statistics     = false,
     termwidth      = nil,
+    keepnames      = true,
   },
   paths = {
     names_dir           = "names",
@@ -611,6 +612,7 @@ local option_spec = {
         return w
       end,
     },
+    keepnames       = { in_t = boolean_t, },
   },
   paths = {
     names_dir           = { in_t = string_t, },
@@ -741,7 +743,7 @@ local formatters = {
     anon_sequence   = { false, format_list    },
     color_callback  = { false, format_string  },
     definer         = { false, format_string  },
-    fontloader      = { false, format_string  },
+    fontloader      = { true, format_string  },
     log_level       = { false, format_integer },
     resolver        = { false, format_string  },
   },
@@ -994,7 +996,7 @@ local function dump ()
   end
   if next(confdata) then
     iowrite (stringformat (conf_header,
-                           osdate ("%Y-%m-d %H:%M:%S", os.time ())))
+                           osdate ("%Y-%m-%d %H:%M:%S", os.time ())))
     iowrite (tableconcat (confdata, "\n"))
     iowrite (conf_footer)
   end
