@@ -67,6 +67,9 @@ local function serialize_dom(parser, current,level, output)
     for k, v in pairs(attr) do
       t[#t+1] = string.format("%s='%s'", k, escape_attr(v))
     end
+    -- sort attributes alphabetically. this will ensure that
+    -- their order will not change between several executions of dom:serialize()
+    table.sort(t)
     if #t == 0 then return "" end
     -- add space before attributes
     return " " .. table.concat(t, " ")
@@ -115,7 +118,12 @@ local function serialize_dom(parser, current,level, output)
     attributes["_text"] = nil
   elseif xtype == "DECL" and name =="xml" then
     -- the xml declaration attributes must be in a correct order
-    insert("<?xml version='%s' encoding='%s' ?>", attributes.version, attributes.encoding)
+    local encoding = attributes.encoding or "utf-8"
+    insert("<?xml version='%s' encoding='%s' ?>", attributes.version, encoding)
+    return output
+  elseif xtype == "CDATA" then
+    -- return content unescaped
+    insert("<![CDATA[%s]]>", text_content)
     return output
   end
 
@@ -226,6 +234,7 @@ local parse = function(
     )
     local el = self
     if self:is_element(el) then
+      el._attr = el._attr or {}
       el._attr[name] = value
       return true
     end
@@ -383,6 +392,17 @@ local parse = function(
     return false, msg
   end
 
+  -- restore correct links to parent elements
+  local function fix_parents(el)
+    for k,v in ipairs(el._children or {}) do
+      if v:is_element() then
+        v._parent = el
+        fix_parents(v)
+      end
+    end
+  end
+
+
   --- Add child node to the current node
   function DOM_Object:add_child_node( 
     child, --- element to be inserted as a current node child
@@ -390,6 +410,7 @@ local parse = function(
     )
     local parent = self
     child._parent = parent
+    fix_parents(child)
     if position then
       table.insert(parent._children, position, child)
     else
