@@ -1,12 +1,13 @@
-# $Id: TLPDB.pm 53076 2019-12-10 06:20:44Z preining $
+# $Id: TLPDB.pm 63068 2022-04-18 05:58:07Z preining $
 # TeXLive::TLPDB.pm - tlpdb plain text database files.
-# Copyright 2007-2019 Norbert Preining
+# Copyright 2007-2021 Norbert Preining
 # This file is licensed under the GNU General Public License version 2
 # or any later version.
 
+use strict; use warnings;
 package TeXLive::TLPDB;
 
-my $svnrev = '$Revision: 53076 $';
+my $svnrev = '$Revision: 63068 $';
 my $_modulerevision = ($svnrev =~ m/: ([0-9]+) /) ? $1 : "unknown";
 sub module_revision { return $_modulerevision; }
 
@@ -14,7 +15,7 @@ sub module_revision { return $_modulerevision; }
 
 =head1 NAME
 
-C<TeXLive::TLPDB> -- A database of TeX Live Packages
+C<TeXLive::TLPDB> -- TeX Live Package Database (C<texlive.tlpdb>) module
 
 =head1 SYNOPSIS
 
@@ -98,7 +99,8 @@ use TeXLive::TLConfig qw($CategoriesRegexp $DefaultCategory $InfraLocation
 use TeXLive::TLCrypto;
 use TeXLive::TLPOBJ;
 use TeXLive::TLUtils qw(dirname mkdirhier member win32 info log debug ddebug
-                        tlwarn basename download_file merge_into tldie system_pipe);
+                        tlwarn basename download_file merge_into tldie
+                        system_pipe);
 use TeXLive::TLWinGoo;
 
 use Cwd 'abs_path';
@@ -280,6 +282,7 @@ sub from_file {
   my $is_verified = 0;
   # do media detection
   my $rootpath = $self->root;
+  my $media;
   if ($rootpath =~ m,https?://|ftp://,) {
     $media = 'NET';
   } elsif ($rootpath =~ m,$TeXLive::TLUtils::SshURIRegex,) {
@@ -370,57 +373,16 @@ sub from_file {
     #
     # before we open and proceed, verify the downloaded file
     if ($params{'verify'} && $media ne 'local_uncompressed') {
-      my ($r, $m) = TeXLive::TLCrypto::verify_checksum($tlpdbfile, "$path.$ChecksumExtension");
-      if ($r == $VS_CHECKSUM_ERROR) {
-        tldie("$0: checksum error when downloading $tlpdbfile from $path: $m\n");
-      } elsif ($r == $VS_SIGNATURE_ERROR) {
-        tldie("$0: signature verification error of $tlpdbfile from $path: $m\n");
-      } elsif ($r == $VS_CONNECTION_ERROR) {
-        tldie("$0: cannot download: $m\n");
-      } elsif ($r == $VS_UNSIGNED) {
-        debug("$0: remote database checksum is not signed, continuing anyway: $m\n");
-        $self->verification_status($r);
-      } elsif ($r == $VS_GPG_UNAVAILABLE) {
-        debug("$0: TLPDB: no gpg available, continuing anyway!\n");
-        $self->verification_status($r);
-      } elsif ($r == $VS_PUBKEY_MISSING) {
-        debug("$0: TLPDB: pubkey missing, continuing anyway!\n");
-        $self->verification_status($r);
-      } elsif ($r == $VS_VERIFIED) {
-        $is_verified = 1;
-        $self->verification_status($r);
-      } else {
-        tldie("$0: unexpected return value from verify_checksum: $r\n");
-      }
+      my ($verified, $status) = TeXLive::TLCrypto::verify_checksum_and_check_return($tlpdbfile, $path);
+      $is_verified = $verified;
+      $self->verification_status($status);
     }
     open($retfh, "<$tlpdbfile") || die "$0: open($tlpdbfile) failed: $!";
   } else {
     if ($params{'verify'} && $media ne 'local_uncompressed') {
-      my ($r, $m) = TeXLive::TLCrypto::verify_checksum($path, "$path.$ChecksumExtension");
-      if ($r == $VS_CHECKSUM_ERROR) {
-        tldie("$0: checksum error when downloading $path from $path: $m\n");
-      } elsif ($r == $VS_SIGNATURE_ERROR) {
-        tldie("$0: signature verification error of $path from $path: $m\n");
-      } elsif ($r == $VS_CONNECTION_ERROR) {
-        tldie("$0: cannot download: $m\n");
-      } elsif ($r == $VS_UNSIGNED) {
-        debug("$0: remote database checksum is not signed, continuing anyway!\n");
-        $self->verification_status($r);
-      } elsif ($r == $VS_GPG_UNAVAILABLE) {
-        debug("$0: TLPDB: no gpg available, continuing anyway!\n");
-        $self->verification_status($r);
-      } elsif ($r == $VS_PUBKEY_MISSING) {
-        debug("$0: TLPDB: pubkey missing, continuing anyway!\n");
-        $self->verification_status($r);
-      } elsif ($r == $VS_EXPKEYSIG) {
-        debug("$0: TLPDB: signature verified, but key expired, continuing anyway!\n");
-        $self->verification_status($r);
-      } elsif ($r == $VS_VERIFIED) {
-        $is_verified = 1;
-        $self->verification_status($r);
-      } else {
-        tldie("$0: unexpected return value from verify_checksum: $r\n");
-      }
+      my ($verified, $status) = TeXLive::TLCrypto::verify_checksum_and_check_return($path, $path);
+      $is_verified = $verified;
+      $self->verification_status($status);
     }
     open(TMP, "<$path") || die "$0: open($path) failed: $!";
     $retfh = \*TMP;
@@ -463,9 +425,10 @@ sub writeout {
     tlwarn("TLPDB: cannot writeout a virtual tlpdb\n");
     return 0;
   }
-  my $fd = (@_ ? $_[0] : STDOUT);
+  my $fd = (@_ ? $_[0] : *STDOUT);
   foreach (sort keys %{$self->{'tlps'}}) {
-    ddebug("writeout: tlpname=$_  ", $self->{'tlps'}{$_}->name, "\n");
+    TeXLive::TLUtils::dddebug("writeout: tlpname=$_  ",
+                              $self->{'tlps'}{$_}->name, "\n");
     $self->{'tlps'}{$_}->writeout($fd);
     print $fd "\n";
   }
@@ -1029,7 +992,7 @@ available architectures as packages with revision number -1.
 
 sub generate_packagelist {
   my $self = shift;
-  my $fd = (@_ ? $_[0] : STDOUT);
+  my $fd = (@_ ? $_[0] : *STDOUT);
   foreach (sort $self->list_packages) {
     print $fd $self->get_package($_)->name, " ",
               $self->get_package($_)->revision, "\n";
@@ -1056,7 +1019,7 @@ sub generate_listfiles {
     $destdir = TeXLive::TLPDB->listdir;
   }
   foreach (sort $self->list_package) {
-    $tlp = $self->get_package($_);
+    my $tlp = $self->get_package($_);
     $self->_generate_listfile($tlp, $destdir);
   }
 }
@@ -1504,7 +1467,7 @@ sub sizes_of_packages_with_deps {
 sub _sizes_of_packages {
   my ($self, $with_deps, $opt_src, $opt_doc, $arch_list_ref, @packs) = @_;
   @packs || ( @packs = $self->list_packages() );
-  my @expacks;
+  my @exppacks;
   if ($with_deps) {
     # don't expand collection->collection dependencies
     #@exppacks = $self->expand_dependencies('-no-collections', $self, @packs);
@@ -1662,7 +1625,8 @@ sub install_package_files {
     my ($tlpobjfile, $anotherfile) = <$tmpdir/tlpkg/tlpobj/*.tlpobj>;
     if (defined($anotherfile)) {
       # we found several tlpobj files, that is not allowed, stop
-      tlwarn("TLPDB::install_package_files: several tlpobj files in $what in tlpkg/tlpobj/, stopping!\n");
+      tlwarn("TLPDB::install_package_files: several tlpobj files "
+             . "($tlpobjfile, $anotherfile) in tlpkg/tlpobj/, stopping!\n");
       next;
     }
     # - read the tlpobj from there
@@ -1690,8 +1654,10 @@ sub install_package_files {
     @installfiles = map { s!^$RelocPrefix/!!; $_; } @installfiles;
     # if the first argument of _install_data is scalar, it is the
     # place from where files should be installed
-    if (!_install_data ($tmpdir, \@installfiles, $reloc, \@installfiles, $self)) {
-      tlwarn("TLPDB::install_package_files: couldn't install $what!\n"); 
+    if (!_install_data ($tmpdir, \@installfiles, $reloc, \@installfiles,
+                        $self)) {
+      tlwarn("TLPDB::install_package_files: couldn't install_data files: "
+             . "@installfiles\n"); 
       next;
     }
     if ($reloc) {
@@ -1816,11 +1782,20 @@ sub not_virtual_install_package {
       }
     }
     my $media = $self->media;
+    my $container_is_versioned = 0;
     if ($media eq 'local_uncompressed') {
       $container = \@installfiles;
     } elsif ($media eq 'local_compressed') {
       for my $ext (map { $Compressors{$_}{'extension'} } keys %Compressors) {
-        if (-r "$root/$Archive/$pkg.tar.$ext") {
+        # request versioned containers when local (i.e., ISO image),
+        # since the unversioned symlinks cannot be dereferenced
+        # on Windows.
+        my $rev = $tlpobj->revision;
+        if (-r "$root/$Archive/$pkg.r$rev.tar.$ext") {
+          $container_is_versioned = 1;
+          $container = "$root/$Archive/$pkg.r$rev.tar.$ext";
+        } elsif (-r "$root/$Archive/$pkg.tar.$ext") {
+          $container_is_versioned = 0;
           $container = "$root/$Archive/$pkg.tar.$ext";
         }
       }
@@ -1830,8 +1805,11 @@ sub not_virtual_install_package {
         return(0);
       }
     } elsif (&media eq 'NET') {
+      # Since the NET server cannot be a Windows machine,
+      # ok to request the unversioned file.
       $container = "$root/$Archive/$pkg.tar."
                    . $Compressors{$DefaultCompressorFormat}{'extension'};
+      $container_is_versioned = 0;
     }
     my $container_str = ref $container eq "ARRAY"
                         ? "[" . join (" ", @$container) . "]" : $container;
@@ -1857,14 +1835,22 @@ sub not_virtual_install_package {
       # - there are actually src/doc files present
       if ($container_src_split && $opt_src && $tlpobj->srcfiles) {
         my $srccontainer = $container;
-        $srccontainer =~ s/\.tar\.$CompressorExtRegexp$/.source.tar.$1/;
+        if ($container_is_versioned) {
+          $srccontainer =~ s/\.(r[0-9]*)\.tar\.$CompressorExtRegexp$/.source.$1.tar.$2/;
+        } else {
+          $srccontainer =~ s/\.tar\.$CompressorExtRegexp$/.source.tar.$1/;
+        }
         $self->_install_data($srccontainer, $reloc, \@installfiles, $totlpdb,
                       $tlpobj->srccontainersize, $tlpobj->srccontainerchecksum)
           || return(0);
       }
       if ($container_doc_split && $real_opt_doc && $tlpobj->docfiles) {
         my $doccontainer = $container;
-        $doccontainer =~ s/\.tar\.$CompressorExtRegexp$/.doc.tar.$1/;
+        if ($container_is_versioned) {
+          $doccontainer =~ s/\.(r[0-9]*)\.tar\.$CompressorExtRegexp$/.doc.$1.tar.$2/;
+        } else {
+          $doccontainer =~ s/\.tar\.$CompressorExtRegexp$/.doc.tar.$1/;
+        }
         $self->_install_data($doccontainer, $reloc, \@installfiles,
             $totlpdb, $tlpobj->doccontainersize, $tlpobj->doccontainerchecksum)
           || return(0);
@@ -1993,10 +1979,10 @@ sub _install_data {
     }
     my $ww = ($whatsize || "<unset>");
     my $ss = ($whatcheck || "<unset>");
-    debug("tlpdb:_install_data: what=$what, target=$target, size=$ww, checksum=$ss, tmpdir=$tempdir\n");
+    debug("TLPDB::_install_data: what=$what, target=$target, size=$ww, checksum=$ss, tmpdir=$tempdir\n");
     my ($ret, $pkg) = TeXLive::TLUtils::unpack($what, $target, 'size' => $whatsize, 'checksum' => $whatcheck, 'tmpdir' => $tempdir);
     if (!$ret) {
-      tlwarn("TLPDB::_install_package: $pkg\n");
+      tlwarn("TLPDB::_install_data: $pkg for $what\n"); # $pkg is error msg
       return(0);
     }
     # remove the $pkg.tlpobj, we recreate it anyway again
@@ -2004,7 +1990,7 @@ sub _install_data {
       if (-r "$target/tlpkg/tlpobj/$pkg.tlpobj");
     return(1);
   } else {
-    tlwarn("TLPDB::_install_package: don't know how to install $what\n");
+    tlwarn("TLPDB::_install_data: don't know how to install $what\n");
     return(0);
   }
 }
@@ -3000,7 +2986,7 @@ C<Master/tlpkg/doc/>.
 =head1 AUTHORS AND COPYRIGHT
 
 This script and its documentation were written for the TeX Live
-distribution (L<http://tug.org/texlive>) and both are licensed under the
+distribution (L<https://tug.org/texlive>) and both are licensed under the
 GNU General Public License Version 2 or later.
 
 =cut
